/*
 * CustomLoggerFactory.java
 *
 * Version 1.2
 *
 * Copyright (c) Kieran Greer
 *
 */

package org.jlog2;


import org.jlog2.util.*;

import java.util.*;


/**
 * This class is an implementation of the logger factory to return the default
 * logger functionality. You can implement your own version instead.
 * @author Kieran Greer
 */
public class CustomLoggerFactory extends LoggerFactory
{
    /**
     * Creates a new instance of CustomLoggerFactory.
     * @throws java.lang.Exception any error.
     */
    public CustomLoggerFactory() throws Exception
    {
        initialiseValues();
    }
    
    /**
     * Initialise some values.
     * @throws java.lang.Exception any error.
     */
    private void initialiseValues() throws Exception
    {
        logChannels = new Hashtable();
        debugChannels = new Hashtable();

        //load properties and default channels
        propsParser = new PropertiesParser();
        propsParser.initialiseChannels(logChannels, debugChannels);
        
        if (propsParser.getProperties().containsKey(LogConst.LOGLEVEL))
        {
            logLevel = propsParser.getProperties().getProperty(LogConst.LOGLEVEL);
        }
        else
        {
            logLevel = LoggerHandler.INFO;
        }

        if (propsParser.getProperties().containsKey(LogConst.DEBUGLEVEL))
        {
            debugLevel = propsParser.getProperties().getProperty(LogConst.DEBUGLEVEL);
        }
        else
        {
            debugLevel = LoggerHandler.INFO;
        }
    }

    /**
     * Create a new instance of Logger. Every call creates a new instance,
     * but the channels to write the messages to might be duplicated.
     * @param loggerID the id to identify one logger from another.
     * @return the newly created logger. 
     */
    public Logger getLoggerInstance(String loggerID)
    {
        Logger newLogger;                   //the new logger
        
        newLogger = new Logger(loggerID);
        return newLogger;
    }

    /**
     * Create and store any new log channels associated with the new log id.
     * @return a list of all log channels associated with the logger.
     * This default implementation simply returns every channel that has been 
     * added and is the same as calling LoggerFactory.getAllLogChannels.
     */
    protected Vector getLogChannels()
    {
        return new Vector(logChannels.values());
    }

    /**
     * Create and store any new debug channels associated with the new log id.
     * @return a list of all debug channels associated with the logger.
     * This default implementation simply returns every channel that has been 
     * added and is the same as calling LoggerFactory.getAllDebugChannels.
     */
    protected Vector getDebugChannels()
    {
        return new Vector(debugChannels.values());
    }
}
