/*
 * Logger.java
 *
 * Created on 16 May 2009
 *
 * Version 1.5
 *
 * Copyright (c) Kieran Greer
 *
 */

package org.jlog2;


import org.jlog2.exception.WriteMessageException;
import org.jlog2.channel.*;

import java.util.*;


/**
 * This class stores log channels appropriate for each Logger instance and
 * provides the methods to log messages to those channels.
 * @author Kieran Greer
 */
public class Logger
{
    /** Separator to separate the different mesage parts */
    private static final String SEP = ": ";


    /** True if log debug trace as well */
    private boolean debug;

    /** The classname of the class that uses the logger */
    private String className;

    /** The unique id to identify this logger, for example the related classname */
    private String loggerID;
    
    /**
     * Create a new instance of Logger.
     * @param thisLoggerID the unique logger id.
     */
    public Logger(String thisLoggerID)
    {
        loggerID = thisLoggerID;
        className = thisLoggerID;
        initialiseValues();
    }

    /**
     * Create a new instance of Logger.
     * @param thisLoggerID the unique logger id.
     * @param thisClassName the classname of the class that uses this logger if
     * different from the logger id.
     */
    public Logger(String thisLoggerID, String thisClassName)
    {
        loggerID = thisLoggerID;
        className = thisClassName;
        initialiseValues();
    }

    /** Initialise some values */
    private void initialiseValues()
    {
        debug = false;
    }

    /**
     * Set a value indicating if the debug channels should be used.
     * @param thisDebug true if log a debug trace, false otherwise.
     */
    public void setDebug(boolean thisDebug)
    {
        debug = thisDebug;
    }

    /**
     * Get the value indicating if the debug channels are being used.
     * @return the value of debug.
     */
    public boolean getDebug()
    {
        return debug;
    }
    
    /**
     * Get the unique id that created this logger.
     * @return the value of loggerID.
     */
    public String getLoggerID()
    {
        return loggerID;
    }

    /**
     * Log the message on the appropriate channels.
     * @param level the log level for the message, error, warning, etc.
     * @param methodName the name of the method in the class logging the message.
     * @param message the message to log.
     * @throws java.lang.Exception any error.
     */
    public void logMessage(String level, String methodName, String message) throws Exception
    {
        int i;
        String wholeMessage;                        //the whole message
        Vector logChannels;                         //all log channels
        
        wholeMessage = ((new java.util.Date()).toString() + SEP + level + SEP);
        wholeMessage += (className + SEP + methodName + SEP + message);

        logChannels = LoggerFactory.getInstance().getLogChannels();
        if (canLog(level) && (logChannels != null))
        {
            for (i = 0; i < logChannels.size(); i++)
            {
                ((LogChannel)logChannels.elementAt(i)).writeMessage(wholeMessage);
            }
        }
    }

    /**
     * Debug the message on the appropriate channels.
     * @param level the log level for the message, error, warning, etc.
     * @param methodName the name of the method in the class logging the message.
     * @param message the message to log.
     * @throws java.lang.Exception any error.
     */
    public void debugMessage(String level, String methodName, String message) throws Exception
    {
        int i;
        String wholeMessage;                            //the whole message
        Vector debugChannels;                           //all debug channels
        
        wholeMessage = ((new java.util.Date()).toString() + SEP + level + SEP);
        wholeMessage += (className + SEP + methodName + SEP + message);

        debugChannels = LoggerFactory.getInstance().getDebugChannels();
        if (debug && canDebug(level) && (debugChannels != null))
        {
            for (i = 0; i < debugChannels.size(); i++)
            {
                ((LogChannel)debugChannels.elementAt(i)).writeMessage(wholeMessage);
            }
        }
    }

    /**
     * Log the error on the appropriate channels.
     * @param level the log level for the message, error, warning, etc.
     * @param error the error to log.
     * @throws java.lang.Exception any error.
     */
    public void logError(String level, Exception error) throws Exception
    {
        int i;
        String errorMessage;                        //the error message
        Vector logChannels;                         //all log channels
        StackTraceElement[] stackTrace;             //stack trace
        
        logChannels = LoggerFactory.getInstance().getLogChannels();
        if (canLog(level) && (logChannels != null))
        {
            errorMessage = error.getLocalizedMessage();
            if (!(error instanceof WriteMessageException))
            {
                stackTrace = error.getStackTrace();
                for (i = 0; i < stackTrace.length; i++)
                {
                    if (stackTrace[i] != null)
                    {
                        errorMessage += ("\n" + stackTrace[i].toString());
                    }
                }
            }
                
            for (i = 0; i < logChannels.size(); i++)
            {
                ((LogChannel)logChannels.elementAt(i)).writeError(errorMessage);
            }
        }
    }

    /**
     * Debug the error on the appropriate channels.
     * @param level the log level for the message, error, warning, etc.
     * @param error the error to debug.
     * @throws java.lang.Exception any error.
     */
    public void debugError(String level, Exception error) throws Exception
    {
        int i;
        Vector debugChannels;                           //all debug channels
        
        debugChannels = LoggerFactory.getInstance().getDebugChannels();
        if (debug && canDebug(level) && (debugChannels != null))
        {
            for (i = 0; i < debugChannels.size(); i++)
            {
                if (error instanceof WriteMessageException)
                {
                    ((LogChannel)debugChannels.elementAt(i)).writeError(error.getLocalizedMessage());
                }
                else
                {
                    ((LogChannel)debugChannels.elementAt(i)).writeError(error.toString());
                }
            }
        }
    }

    /**
     * Return true if the logger can log to the specified level.
     * @param messageLevel the level to log to.
     * @return true if can log at this level.
     */
    private boolean canLog(String messageLevel)
    {
        return LoggerHandler.levelOK(messageLevel, LoggerFactory.getLogLevel());
    }

    /**
     * Return true if the logger can debug to the specified level.
     * @param messageLevel the level to log to.
     * @return true if can log at this level.
     */
    private boolean canDebug(String messageLevel)
    {
        return LoggerHandler.levelOK(messageLevel, LoggerFactory.getDebugLevel());
    }
}
