/*
 * LoggerFactory.java
 *
 * Created on 16 May 2009
 *
 * Version 1.4
 *
 * Copyright (c) Kieran Greer
 *
 */

package org.jlog2;


import org.jlog2.util.*;

import java.util.*;


/**
 * This abstract class must be implemented to return logger instances for each specific logger type.
 * @author Kieran Greer
 */
public abstract class LoggerFactory
{
    /** Logging properties */
    protected static PropertiesParser propsParser;      
      
    
    /**
     * The log channels mapped to the appropriate logger ids. Keys are the logger
     * ids of type String and values are the log channels of type LogChannels.
     */
    protected Hashtable logChannels;

    /**
     * The debug channels mapped to the appropriate logger ids. Keys are the logger
     * ids of type String and values are the debug channels of type LogChannels.
     */
    protected Hashtable debugChannels;
    
    /** The highest level to log messages at */
    protected static String logLevel;

    /** The highest level to log debug errors at */
    protected static String debugLevel;

    /** Instance of the logger factory */
    private static LoggerFactory instance = null;
    
    
    /**
     * Get the created instance of the logger factory.
     * @return the value of instance.
     */
    public static LoggerFactory getInstance()
    {
        return instance;
    }

    /**
     * Set the instance of the logger factory to be used.
     * @param thisLoggerFactory the logger factory to use.
     */
    public static void setLoggerFactory(LoggerFactory thisLoggerFactory)
    {
        if (instance == null) instance = thisLoggerFactory;
    }

    /**
     * Create a new instance of Logger.
     * @param loggerID the id to identify one logger from another.
     * @return the newly created logger. 
     */
    public abstract Logger getLoggerInstance(String loggerID);
    
    /**
     * Get the list of log channels.
     * @return a list of all log channels.
     */
    protected abstract Vector getLogChannels();

    /**
     * Get the list of debug channels.
     * @return a list of all debug channels.
     */
    protected abstract Vector getDebugChannels();
    
    
    /**
     * Reset the channels by reading the properties file again and adding any new
     * standard ones, or updating the values of the existing ones.
     * @throws java.lang.Exception any error.
     */
    public void resetChannels() throws Exception
    {
        propsParser = new PropertiesParser();
        propsParser.resetChannels(logChannels, debugChannels);
    }

    /**
     * Get all channels to write the log messages to, as a list.
     * @return a list of all default log channels.
     */
    public Hashtable getAllLogChannels()
    {
        return logChannels;
    }

    /**
     * Get all channels to write the debug messages to, as a list.
     * @return a list of all default debug channels.
     */
    public Hashtable getAllDebugChannels()
    {
        return debugChannels;
    }
    
    /**
     * Set the level to log messages to.
     * @param theLogLevel the level to log messages to.
     */
    public static void setLogLevel(String theLogLevel)
    {
        logLevel = theLogLevel;
    }
    
    /**
     * Get the level to log messages to.
     * @return the value of logLevel.
     */
    public static String getLogLevel()
    {
        return logLevel;
    }
    
    /**
     * Set the level to debug messages to.
     * @param theDebugLevel the level to debug messages to.
     */
    public static void setDebugLevel(String theDebugLevel)
    {
        debugLevel = theDebugLevel;
    }
    
    /**
     * Get the level to debug messages to.
     * @return the value of debugLevel.
     */
    public static String getDebugLevel()
    {
        return debugLevel;
    }
}
