/*
 * FileChannel.java
 *
 * Created on 17 May 2009
 *
 * Version 1.2
 *
 * Copyright (c) Kieran Greer
 *
 */

package org.jlog2.channel;

import java.io.*;
import java.util.*;


/**
 * This class provides a channel to log messages to a file.
 * @author Kieran Greer
 */
public class FileChannel extends LogChannel
{
    private static final String BACKUPSEP = "_";

    /** True if just initialising the channel */
    private boolean initialise;

    /** The full path of the file */
    private String filePath;

    /** The maximum allowed size for the file */
    private int maxSize;

    /** The current size for the file */
    private int currentSize;
    
    /** Maximum number of backups to store */
    private int backupNumber;

    /** List of backup indexes */
    private Vector backups;

    /** The log file stream */
    private PrintStream logStream = null;


    /**
     * Create a new instance of FileChannel.
     * @param theFilePath the path of the file to create.
     * @param theMaxSize the maximum allowed size of the file.
     * @param theBackupNumber the maximum number of previous files to keep.
     * @throws java.lang.Exception any error.
     */
    public FileChannel(String theFilePath, int theMaxSize, int theBackupNumber)
            throws Exception
    {
        filePath = theFilePath;
        maxSize = theMaxSize;
        backupNumber = theBackupNumber;
        backups = new Vector();

        initialise = true;
        createLogChannel();
        initialise = false;
    }

    /**
     * Reset the FileChannel's file size and number of backups.
     * @param theMaxSize the maximum allowed size of the file.
     * @param theBackupNumber the maximum number of previous files to keep.
     * @throws java.lang.Exception any error.
     */
    public void resetChannel(int theMaxSize, int theBackupNumber) throws Exception
    {
        maxSize = theMaxSize;
        backupNumber = theBackupNumber;
    }
    
    /**
     * Create the file channel.
     * @throws java.lang.Exception any error.
     */
    protected void createLogChannel() throws Exception
    {
        int backupInt;                              //stored backup number
        int nextChar;                               //next character
        String backupFilePath;                      //backup file path
        File bkupFile;                              //old backup file
        FileReader fileReader;                      //file reader
        FileWriter fileWriter;                      //file writer
        
        currentSize = 0;
        if (logStream != null)
        {
            logStream.close();
            logStream = null;
        }

        if (!initialise && (backupNumber > 0))
        {
            if (backups.size() >= backupNumber)
            {
                backupInt = ((Integer)backups.firstElement()).intValue();
                backupFilePath = (filePath + BACKUPSEP + String.valueOf(backupInt));
                bkupFile = new File(backupFilePath);
                bkupFile.delete();

                backups.removeElementAt(0);
            }

            if (backups.size() > 0)
            {
                backupInt = ((Integer)backups.lastElement()).intValue();
                backupInt++;
            }
            else
            {
                backupInt = 1;
            }

            backupFilePath = (filePath + BACKUPSEP + String.valueOf(backupInt));
            backups.addElement(new Integer(backupInt));
            
            fileWriter = new FileWriter(backupFilePath);
            fileReader = new FileReader(filePath);

            nextChar = fileReader.read();
            while (nextChar != -1)
            {
                fileWriter.write(nextChar);
                nextChar = fileReader.read();
            }
            
            fileReader.close();
            fileWriter.close();
        }

        logStream = new PrintStream(new FileOutputStream(filePath));
    }

    /**
     * Count the number of lines in the message.
     * @param message the message to parse.
     * @return the number of lines, separated by newline characters.
     */
    private void lineCount(String message)
    {
        int index;                          //index
        String remainingStr;                //String with remaining text
        String separator;                   //the separator


        remainingStr = message;
        separator = "\n";
        index = remainingStr.indexOf(separator);

        if (index >= 0)
        {
            while (index >= 0)
            {
                currentSize++;              
                remainingStr = remainingStr.substring(index, remainingStr.length());
                if (remainingStr.length() > 0) remainingStr = remainingStr.substring(1);

                if (remainingStr.length() > 0)
                    index = remainingStr.indexOf(separator);
                else
                    index = -1;
            }
        }
        else
        {
            currentSize++;
        }
    }
    
    /**
     * Write the message to the file channel.
     * @param message the message to write.
     * @throws java.lang.Exception any error.
     */
    public void writeMessage(String message) throws Exception
    {
        synchronized(this)
        {
            logStream.println(message);
            logStream.flush();

            lineCount(message);
            if (currentSize >= maxSize) createLogChannel();
        }
    }

    /**
     * Write an exception to the file channel.
     * @param message the message to write.
     * @throws java.lang.Exception any error.
     */
    public void writeError(String message) throws Exception
    {
        synchronized(this)
        {
            logStream.println(message);
            logStream.flush();
            
            lineCount(message);
            if (currentSize >= maxSize) createLogChannel();
        }
    }

    /**
     * Cleanup any open channels.
     */
    public void finalize()
    {
        if (logStream != null)
        {
            logStream.flush();
            logStream.close();
        }
    }
}
