/*
 * WriteMessageException.java
 *
 * Created on 16 November 2014.
 *
 * Version 1.0
 *
 * Copyright (c) Kieran Greer
 */


package org.jlog2.exception;


/**
 * This class can be used to throw an exception indicating that the message should be
 * logged only once. If used with the {@link ExceptionHandler} of this package and the 
 * appropriate setting, it will write only the message part and also only use the exception once. 
 * If used once, an {@link EndOfException} exception is then returned instead. This means that 
 * you can catch and write the exception several times, or only once, which might lead to tidier 
 * logging.
 * @author Kieran Greer
 */
public class WriteMessageException extends Exception
{
    /** If true, use the exception in the logger only once */
    private boolean useOnce;
    
    /**
     * Create a new instance of WriteMessageException. This exception is passed through
     * each time, although only the message part gets written to the logger.
     * @param message the error message.
     */
    public WriteMessageException(String message)
    {
        super(message);
        useOnce = false;
    }
    
    /**
     * Create a new instance of WriteMessageException.
     * @param message the error message.
     * @param thisUseOnce if true pass through the logger only once, if false, then pass
     * through the logger each time.
     */
    public WriteMessageException(String message, boolean thisUseOnce)
    {
        super(message);
        useOnce = thisUseOnce;
    }
    
    /**
     * Get the value indicating to use only once.
     * @return the value of useOnce.
     */
    protected boolean getUseOnce()
    {
        return useOnce;
    }
}
