/*
 * CollectionHandler.java
 *
 * Created on 13 December 2010
 *
 * Version 1.2
 *
 * Copyright (c) Kieran Greer
 */

package org.jlog2.util;

import java.util.*;

/**
 * This provides some utility functions for java.util collections or lists, etc.
 * @author Kieran Greer
 */
public class CollectionHandler
{
    /**
     * Clone the vector list and return.
     * @param toClone the list to clone.
     * @return a light copy (clone list only) of the vector.
     */
    public static Vector cloneVector(Vector toClone)
    {
        int i;
        Vector vectorClone;                         //clone of vector

        vectorClone = null;

        if (toClone != null)
        {
            vectorClone = new Vector();
            for (i = 0; i < toClone.size(); i++)
            {
                vectorClone.addElement(toClone.elementAt(i));
            }
        }

        return vectorClone;
    }

    /**
     * Add all values in toAdd to addTo.
     * @param addTo the list to add to.
     * @param toAdd the values to add.
     */
    public static void addAll(Vector addTo, Vector toAdd)
    {
        int i;
        
        if (toAdd != null)
        {
            for (i = 0; i < toAdd.size(); i++)
            {
                addTo.addElement(toAdd.elementAt(i));
            }
        }
    }
    
    /**
     * Add all values in toAdd to addTo, but do not allow duplication of existing values.
     * @param addTo the list to add to.
     * @param toAdd the values to add.
     */
    public static void addAllSet(Vector addTo, Vector toAdd)
    {
        int i;
        
        if (toAdd != null)
        {
            for (i = 0; i < toAdd.size(); i++)
            {
                if (!addTo.contains(toAdd.elementAt(i)))
                    addTo.addElement(toAdd.elementAt(i));
            }
        }
    }

    /**
     * Remove all values in toRemove from removeFrom.
     * @param removeFrom the list to remove from.
     * @param toRemove the list of items to remove.
     */
    public static void removeAll(Vector removeFrom, Vector toRemove)
    {
        int i;
        
        if (toRemove != null)
        {
            for (i = 0; i < toRemove.size(); i++)
            {
                removeFrom.removeElement(toRemove.elementAt(i));
            }
        }
    }
    
    /**
     * Return a list of all elements that are in both list 1 and list 2.
     * @param list1 the first list of indexes.
     * @param list2 the second list of indexes.
     * @return a list of all indexes in both lists.
     */
    public static Vector combinedVector(Vector list1, Vector list2)
    {
        int i, j;
        boolean indexFound;                     //true if index is found
        Integer index1;                         //first index
        Integer index2;                         //second index
        Vector newList;                         //new index list

        newList = (Vector)list2.clone();
        for (i = 0; i < list1.size(); i++)
        {
            indexFound = false;
            index1 = (Integer)list1.elementAt(i);

            for (j = 0; !indexFound && (j < list2.size()); j++)
            {
                index2 = (Integer)list2.elementAt(j);
                indexFound = (index2.intValue() == index1.intValue());
            }

            if (!indexFound) newList.addElement(new Integer(index1.intValue()));
        }

        return newList;
    }

    /**
     * Return a list of all elements that are in list 1 but not in list 2.
     * @param list1 the first list of indexes.
     * @param list2 the second list of indexes.
     * @return a list of all indexes in list 1 but not in list 2.
     */
    public static Vector reciprocalVector(Vector list1, Vector list2)
    {
        int i, j;
        boolean indexFound;                     //true if index is found
        Integer index1;                         //first index
        Integer index2;                         //second index
        Vector newList;                         //new index list

        newList = new Vector();
        for (i = 0; i < list1.size(); i++)
        {
            indexFound = false;
            index1 = (Integer)list1.elementAt(i);

            for (j = 0; !indexFound && (j < list2.size()); j++)
            {
                index2 = (Integer)list2.elementAt(j);
                indexFound = (index2.intValue() == index1.intValue());
            }

            if (!indexFound) newList.addElement(new Integer(index1.intValue()));
        }

        return newList;
    }

    /**
     * Clone the list and return as an array.
     * @param toClone the list to clone.
     * @return a copy of the elements in an array list.
     */
    public static Object[] toArray(Vector toClone)
    {
        int i;
        Object[] objArray;                              //array list

        objArray = null;

        if (toClone != null)
        {
            objArray = new Object[toClone.size()];

            for (i = 0; i < toClone.size(); i++)
            {
                objArray[i] = toClone.elementAt(i);
            }
        }

        return objArray;
    }

    /**
     * Clone the hashtable list and return.
     * @param toClone the list to clone.
     * @return a light copy (clone list only) of the hashtable.
     */
    public static Hashtable cloneHashtable(Hashtable toClone)
    {
        int i;
        Object nextKey;                                 //next key
        Vector allKeys;                                 //all keys
        Hashtable hashClone;                            //clone of hashtable

        hashClone = null;

        if (toClone != null)
        {
            hashClone = new Hashtable();
            allKeys = new Vector(toClone.keySet());
            for (i = 0; i < allKeys.size(); i++)
            {
                nextKey = allKeys.elementAt(i);
                hashClone.put(nextKey, toClone.get(nextKey));
            }
        }

        return hashClone;
    }

    /**
     * Clone the hashtable keys and return as a list.
     * @param toClone the list to clone.
     * @return a light copy (clone list only) of the hashtable keys.
     */
    public static Vector cloneHashKeys(Hashtable toClone)
    {
        int i;
        Object nextKey;                                 //next key
        Vector allKeys;                                 //all keys
        Vector hashClone;                               //clone of hashtable

        hashClone = null;

        if (toClone != null)
        {
            hashClone = new Vector();
            allKeys = new Vector(toClone.keySet());
            for (i = 0; i < allKeys.size(); i++)
            {
                nextKey = allKeys.elementAt(i);
                hashClone.addElement(nextKey);
            }
        }

        return hashClone;
    }

    /**
     * Clone the hashtable values and return as a list.
     * @param toClone the list to clone.
     * @return a light copy (clone list only) of the hashtable values.
     */
    public static Vector cloneHashValues(Hashtable toClone)
    {
        int i;
        Object nextKey;                                 //next key
        Vector allKeys;                                 //all keys
        Vector hashClone;                               //clone of hashtable

        hashClone = null;

        if (toClone != null)
        {
            hashClone = new Vector();
            allKeys = new Vector(toClone.keySet());
            for (i = 0; i < allKeys.size(); i++)
            {
                nextKey = allKeys.elementAt(i);
                hashClone.addElement(toClone.get(nextKey));
            }
        }

        return hashClone;
    }

    /**
     * Add all values in toAdd to addTo.
     * @param addTo the hashtable to add to.
     * @param toAdd the values to add.
     */
    public static void putAll(Hashtable addTo, Hashtable toAdd)
    {
        int i;
        Object nextKey;                                 //next key
        Vector allKeys;                                 //all keys
        
        if (toAdd != null)
        {
            allKeys = new Vector(toAdd.keySet());
            for (i = 0; i < allKeys.size(); i++)
            {
                nextKey = allKeys.elementAt(i);
                addTo.put(nextKey, toAdd.get(nextKey));
            }
        }
    }
}
