/*
 * Monitor.java
 *
 * Created on 08 May 2007
 *
 * Version 1.1.1
 *
 * Copyright (c) 2006 Kieran Greer
 */

package org.jlog2.util;


/**
 * To synchronize threads on.
 * @author Kieran Greer
 */
public class Monitor
{    
    /** An object to synchronize on */
    protected static final Object syncOn;
    
    /** The variable to synchronize on */
    protected static final Monitor theMonitor;
    
    
    /** Creates a new instance of Monitor */
    private Monitor()
    {}
    
    static
    {
        syncOn = new Object();
        theMonitor = new Monitor();
    }
    
    /**
     * Return the monitor instance.
     * @return the value of theMonitor.
     */
    public static Monitor getMonitor()
    {
        return theMonitor;
    }

    /**
     * Get an object to synchronize on.
     * @return the value of syncOn.
     */
    public Object getSyncOn()
    {
        return syncOn;
    }
    
    /**
     * Lock current thread on this monitor.
     */
    public void lock()
    {
        synchronized(theMonitor)
        {
            try
            {
                theMonitor.wait();
            }
            catch (InterruptedException ex)
            {}
        }
    }
    
    /**
     * Release locked threads.
     */
    public void release()
    {
        synchronized(theMonitor)
        {
            try
            {            
                theMonitor.notifyAll();
            }
            catch (Exception ex)
            {}
        }
    }
    
    /**
     * Re-start the thread running depending on its current state.
     * @param theThread the thread to stop.
     */
    public static void stopThread(Thread theThread)
    {
        synchronized(theThread)
        {
            try 
            {
                theThread.wait();
            } 
            catch (InterruptedException ex) {}
        }
    }
    
    /**
     * Re-start the thread running depending on its current state.
     * @param theThread the thread to start.
     * @param startNew true if start a new thread for the first time.
     */
    public static void restartThread(Thread theThread, boolean startNew)
    {
        synchronized(theThread)
        {
            if (startNew && !theThread.isAlive() && !theThread.isInterrupted())
            {
                try
                {
                    theThread.start();
                }
                catch (Exception ex)
                {
                    //if thread terminated then do not flag error
                }
            }
            else if (theThread.isAlive())
            {
                theThread.notify();
            }
        }
    }
}
