/*
 * PropertiesParser.java
 *
 * Created on 18 May 2009
 *
 * Version 1.2
 *
 * Copyright (c) Kieran Greer
 */

package org.jlog2.util;


import org.jlog2.channel.*;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.*;


/**
 * This class acts as a parser of a properties file to configure the logger
 * @author Kieran Greer
 */
public class PropertiesParser
{
    private static final String HEADING = 
"##################################################################################################################" +
"\n#" +
"\n# This file is a configuration file for the logger jlog2. Note that the declared values are not case sensitive." +
"\n#" +
"\n# LOG CHANNELS" +
"\n# The logger allows logging to the default log channels standard output (O), standard error (E), or a file (F)" +
"\n#" +
"\n# Declare the standard channels by specifying the representing characters, for example:" +
"\n#		1. To log to a file and standard output the instruction is: log.channels = FO" +
"\n#		2. To debug to a file and standard error the instruction is: debug.channels = FE" +
"\n#" +
"\n# The logger can then also log to user defined loggers that are written as part of the code" +
"\n# and manually added to the logger when the code is initialised. See the documentation for details on this." +
"\n#" +
"\n#" +
"\n# FILE SIZE" +
"\n# Each log file has a maximum size as specified by the file size property. This is the maximum number of lines" +
"\n# allowed in the log file." +
"\n#" +
"\n# Declare the file sizes as follows:" +
"\n#		1. To log to a file with a maximum number of 100000 lines the instruction is: log.file.size = 100000" +
"\n#		2. To debug to a file with a maximum number of 10000 lines the instruction is: debug.file.size = 10000" +
"\n#" +
"\n# You can also save earlier versions of the log file that would otherwise be overwritten when the maximum size" +
"\n# is reached. The number of previous files that you keep is declared by the backup property." +
"\n#" +
"\n# Declare the number of backup files to keep as follows:" +
"\n#		1. log.file.backup = 5 means to keep 5 earlier versions of the log file. These are always the most recent earlier versions." +
"\n#		2. debug.file.backup = 1 means to keep one earlier version of the debug file. This is always the most recent earlier version." +
"\n#" +
"\n#" +
"\n# LOG LEVELS" +
"\n# Logging can be declared at different levels representing different levels of importance." +
"\n# The levels available in order are: All, Info, Warn, Error, Severe, Message" +
"\n#" +
"\n# Declare the level for each logger as follows:" +
"\n#		1. To log to a level of info or higher (Info, Warn, Error, Severe): log.level = Info" +
"\n#		2. To debug to a level of error or higher (Error, Severe): debug.level = Error" +
"\n#" +
"\n# The following additional conditions apply:" +
"\n#		1. When specifying the level in the config file, the 'All' level specifies to log everything." +
"\n#		2. When declaring the log level in your code, the 'LoggerHandler.MESSAGE' level specifies to log the message" +
"\n#			no matter what level is set in the config file. The 'Message' level means to always output the" +
"\n#			message, even if it is not associated with any specific problem or level, for example." +
"\n#" +
"\n##################################################################################################################";
      
    
    /** The properties object */
    private Properties props;

    
    /**
     * Create a new instance of PropertiesParser.
     * @throws java.lang.Exception any error.
     */
    public PropertiesParser() throws Exception
    {
        loadProperties();
    }
    
    /**
     * Load in the properties file values.
     * @throws java.lang.Exception any error.
     */
    private void loadProperties() throws Exception
    {
        FileInputStream inputFile;                  //input file
        
        inputFile = null;
        props = new Properties();

        try
        {
            inputFile = new FileInputStream(ConfigHandler.getConfigFile());
            props.load(inputFile);
        }
        catch (Exception ex)
        {
            props = new Properties();
        }
        finally
        {
            if (inputFile != null) inputFile.close();
        }
    }

    /**
     * Get the properties that configure the loggers.
     * @return the properties read from a file.
     */
    public Properties getProperties()
    {
        return props;
    }

    /**
     * Initialise the log and debug channels.
     * @param logChannels the log channels object to initialise. Keys are the
     * logger keys while values are the default channels.
     * @param debugChannels the debug channels object to initialise. Keys are the
     * logger keys while values are the default channels.
     * @throws java.lang.Exception any error.
     */
    public void initialiseChannels(Hashtable logChannels, Hashtable debugChannels) throws Exception
    {
        int fileSize;                           //file size
        int backupNumber;                       //number of backup files
        String filePath;                        //file path
        String defaultChannels;                 //default log channels
        String nextProperty;                    //next property
        LogChannel nextChannel;                 //next log channel

        //parse the default channels
        if (props.containsKey(LogConst.LOGCHANNELS))
        {
            defaultChannels = props.getProperty(LogConst.LOGCHANNELS).toLowerCase();

            if (defaultChannels.indexOf(LogConst.STDOUTKEY) >= 0)
            {
                nextChannel = new StdOutChannel();
                logChannels.put(LogConst.STDOUTID, nextChannel);
            }

            if (defaultChannels.indexOf(LogConst.STDERRKEY) >= 0)
            {
                nextChannel = new StdErrChannel();
                logChannels.put(LogConst.STDERRID, nextChannel);
            }

            if (defaultChannels.indexOf(LogConst.FILEKEY) >= 0)
            {
                if (props.containsKey(LogConst.LOGFILEPATH))
                {
                    filePath = (ConfigHandler.getLogFolder() + "/" + 
                            props.getProperty(LogConst.LOGFILEPATH));

                    if (props.containsKey(LogConst.LOGFILESIZE))
                    {
                        nextProperty = props.getProperty(LogConst.LOGFILESIZE);
                        fileSize = Integer.parseInt(nextProperty);
                    }
                    else
                    {
                        fileSize = ConfigHandler.getDefaultFileSize();
                    }

                    if (props.containsKey(LogConst.LOGFILEBACKUP))
                    {
                        nextProperty = props.getProperty(LogConst.LOGFILEBACKUP);
                        backupNumber = Integer.parseInt(nextProperty);
                    }
                    else
                    {
                        backupNumber = 0;
                    }

                    nextChannel = new FileChannel(filePath, fileSize, backupNumber);
                    logChannels.put(LogConst.STDFILEID, nextChannel);
                }
            }
        }

        if (props.containsKey(LogConst.DEBUGCHANNELS))
        {
            defaultChannels = props.getProperty(LogConst.DEBUGCHANNELS).toLowerCase();

            if (defaultChannels.indexOf(LogConst.STDOUTKEY) >= 0)
            {
                nextChannel = new StdOutChannel();
                debugChannels.put(LogConst.STDOUTID, nextChannel);
            }

            if (defaultChannels.indexOf(LogConst.STDERRKEY) >= 0)
            {
                nextChannel = new StdErrChannel();
                debugChannels.put(LogConst.STDERRID, nextChannel);
            }

            if (defaultChannels.indexOf(LogConst.FILEKEY) >= 0)
            {
                if (props.containsKey(LogConst.DEBUGFILEPATH))
                {
                    filePath = (ConfigHandler.getLogFolder() + "/" + 
                            props.getProperty(LogConst.DEBUGFILEPATH));

                    if (props.containsKey(LogConst.DEBUGFILESIZE))
                    {
                        nextProperty = props.getProperty(LogConst.DEBUGFILESIZE);
                        fileSize = Integer.parseInt(nextProperty);
                    }
                    else
                    {
                        fileSize = ConfigHandler.getDefaultFileSize();
                    }

                    if (props.containsKey(LogConst.DEBUGFILEBACKUP))
                    {
                        nextProperty = props.getProperty(LogConst.DEBUGFILEBACKUP);
                        backupNumber = Integer.parseInt(nextProperty);
                    }
                    else
                    {
                        backupNumber = 0;
                    }

                    nextChannel = new FileChannel(filePath, fileSize, backupNumber);
                    debugChannels.put(LogConst.STDFILEID, nextChannel);
                }
            }
        }
    }
    
    /**
     * Reset the log and debug channels.
     * @param logChannels the log channels object to initialise. Keys are the
     * logger keys while values are the default channels.
     * @param debugChannels the debug channels object to initialise. Keys are the
     * logger keys while values are the default channels.
     * @throws java.lang.Exception any error.
     */
    public void resetChannels(Hashtable logChannels, Hashtable debugChannels) throws Exception
    {
        int i;
        int fileSize;                           //file size
        int backupNumber;                       //number of backup files
        String filePath;                        //file path
        String defaultChannels;                 //default log channels
        String nextProperty;                    //next property
        LogChannel nextChannel;                 //next log channel
        
        //parse the default channels
        if (props.containsKey(LogConst.LOGCHANNELS))
        {
            defaultChannels = props.getProperty(LogConst.LOGCHANNELS).toLowerCase();

            if (defaultChannels.indexOf(LogConst.STDOUTKEY) >= 0)
            {
                if (!logChannels.containsKey(LogConst.STDOUTID))
                {
                    nextChannel = new StdOutChannel();
                    logChannels.put(LogConst.STDOUTID, nextChannel);
                }
            }
            else
            {
                logChannels.remove(LogConst.STDOUTID);
            }

            if (defaultChannels.indexOf(LogConst.STDERRKEY) >= 0)
            {
                if (!logChannels.containsKey(LogConst.STDERRID))
                {
                    nextChannel = new StdErrChannel();
                    logChannels.put(LogConst.STDERRID, nextChannel);
                }
            }
            else
            {
                logChannels.remove(LogConst.STDERRID);
            }

            if (defaultChannels.indexOf(LogConst.FILEKEY) >= 0)
            {
                if (props.containsKey(LogConst.LOGFILEPATH))
                {
                    filePath = (ConfigHandler.getLogFolder() + "/" + 
                            props.getProperty(LogConst.LOGFILEPATH));

                    if (props.containsKey(LogConst.LOGFILESIZE))
                    {
                        nextProperty = props.getProperty(LogConst.LOGFILESIZE);
                        fileSize = Integer.parseInt(nextProperty);
                    }
                    else
                    {
                        fileSize = ConfigHandler.getDefaultFileSize();
                    }

                    if (props.containsKey(LogConst.LOGFILEBACKUP))
                    {
                        nextProperty = props.getProperty(LogConst.LOGFILEBACKUP);
                        backupNumber = Integer.parseInt(nextProperty);
                    }
                    else
                    {
                        backupNumber = 0;
                    }

                    if (!logChannels.containsKey(LogConst.STDFILEID))
                    {
                        nextChannel = new FileChannel(filePath, fileSize, backupNumber);
                        logChannels.put(LogConst.STDFILEID, nextChannel);
                    }
                    else
                    {
                        nextChannel = (LogChannel)logChannels.get(LogConst.STDFILEID);
                        ((FileChannel)nextChannel).resetChannel(fileSize, backupNumber);
                    }
                }
            }
            else
            {
                logChannels.remove(LogConst.STDFILEID);
            }
        }

        if (props.containsKey(LogConst.DEBUGCHANNELS))
        {
            defaultChannels = props.getProperty(LogConst.DEBUGCHANNELS).toLowerCase();

            if (defaultChannels.indexOf(LogConst.STDOUTKEY) >= 0)
            {
                if (!debugChannels.containsKey(LogConst.STDOUTID))
                {
                    nextChannel = new StdOutChannel();
                    debugChannels.put(LogConst.STDOUTID, nextChannel);
                }
            }
            else
            {
                debugChannels.remove(LogConst.STDOUTID);
            }

            if (defaultChannels.indexOf(LogConst.STDERRKEY) >= 0)
            {
                if (!debugChannels.containsKey(LogConst.STDERRID))
                {
                    nextChannel = new StdErrChannel();
                    debugChannels.put(LogConst.STDERRID, nextChannel);
                }
            }
            else
            {
                debugChannels.remove(LogConst.STDERRID);
            }

            if (defaultChannels.indexOf(LogConst.FILEKEY) >= 0)
            {
                if (props.containsKey(LogConst.DEBUGFILEPATH))
                {
                    filePath = (ConfigHandler.getLogFolder() + "/" + 
                            props.getProperty(LogConst.DEBUGFILEPATH));

                    if (props.containsKey(LogConst.DEBUGFILESIZE))
                    {
                        nextProperty = props.getProperty(LogConst.DEBUGFILESIZE);
                        fileSize = Integer.parseInt(nextProperty);
                    }
                    else
                    {
                        fileSize = ConfigHandler.getDefaultFileSize();
                    }

                    if (props.containsKey(LogConst.DEBUGFILEBACKUP))
                    {
                        nextProperty = props.getProperty(LogConst.DEBUGFILEBACKUP);
                        backupNumber = Integer.parseInt(nextProperty);
                    }
                    else
                    {
                        backupNumber = 0;
                    }
                   
                    if (!debugChannels.containsKey(LogConst.STDFILEID))
                    {
                        nextChannel = new FileChannel(filePath, fileSize, backupNumber);
                        debugChannels.put(LogConst.STDFILEID, nextChannel);
                    }
                    else
                    {
                        nextChannel = (LogChannel)debugChannels.get(LogConst.STDFILEID);
                        ((FileChannel)nextChannel).resetChannel(fileSize, backupNumber);
                    }
                }
            }
            else
            {
                debugChannels.remove(LogConst.STDFILEID);
            }
        }
    }
    
    /**
     * Write the properties to the properties file at the default location.
     * Then need to load in again to change the currently used property settings.
     * @param newProps the new set of properties.
     * @throws java.lang.Exception any error.
     */
    public void writeToFile(Properties newProps) throws Exception
    {        
        String logProps;                                //log properties
        String nextProperty;                            //next property
        FileOutputStream outputFile;                    //output file
        
        outputFile = null;
        try
        {
            outputFile = new FileOutputStream(ConfigHandler.getConfigFile());            
            logProps = (HEADING + "\n\n");
            
            if (newProps.containsKey(LogConst.LOGCHANNELS))
            {
                nextProperty = newProps.getProperty(LogConst.LOGCHANNELS);
                logProps += (LogConst.LOGCHANNELS + " = " + nextProperty.trim() + "\n");
            }
            
            if (newProps.containsKey(LogConst.DEBUGCHANNELS))
            {
                nextProperty = newProps.getProperty(LogConst.DEBUGCHANNELS);
                logProps += (LogConst.DEBUGCHANNELS + " = " + nextProperty.trim() + "\n");
            }
                
            if (newProps.containsKey(LogConst.LOGLEVEL))
            {
                nextProperty = newProps.getProperty(LogConst.LOGLEVEL);
                logProps += (LogConst.LOGLEVEL + " = " + nextProperty.trim() + "\n");
            }
            
            if (newProps.containsKey(LogConst.LOGFILEPATH))
            {
                nextProperty = newProps.getProperty(LogConst.LOGFILEPATH);
                logProps += (LogConst.LOGFILEPATH + " = " + nextProperty.trim() + "\n");
            }

            if (newProps.containsKey(LogConst.LOGFILESIZE))
            {
                nextProperty = newProps.getProperty(LogConst.LOGFILESIZE);
                logProps += (LogConst.LOGFILESIZE + " = " + nextProperty.trim() + "\n");
            }

            if (newProps.containsKey(LogConst.LOGFILEBACKUP))
            {
                nextProperty = newProps.getProperty(LogConst.LOGFILEBACKUP);
                logProps += (LogConst.LOGFILEBACKUP + " = " + nextProperty.trim() + "\n");
            }

            if (newProps.containsKey(LogConst.DEBUGLEVEL))
            {
                nextProperty = newProps.getProperty(LogConst.DEBUGLEVEL);
                logProps += (LogConst.DEBUGLEVEL + " = " + nextProperty.trim() + "\n");
            }
            
            if (newProps.containsKey(LogConst.DEBUGFILEPATH))
            {
                nextProperty = newProps.getProperty(LogConst.DEBUGFILEPATH);
                logProps += (LogConst.DEBUGFILEPATH + " = " + nextProperty.trim() + "\n");
            }

            if (newProps.containsKey(LogConst.DEBUGFILESIZE))
            {
                nextProperty = newProps.getProperty(LogConst.DEBUGFILESIZE);
                logProps += (LogConst.DEBUGFILESIZE + " = " + nextProperty.trim() + "\n");
            }

            if (newProps.containsKey(LogConst.DEBUGFILEBACKUP))
            {
                nextProperty = newProps.getProperty(LogConst.DEBUGFILEBACKUP);
                logProps += (LogConst.DEBUGFILEBACKUP + " = " + nextProperty.trim() + "\n");
            }
            
            outputFile.write(logProps.getBytes());
        }
        finally
        {
            if (outputFile != null) outputFile.close();
        }
    }
}
