/*
 * StringHandler.java
 *
 * Created on 12 May 2009
 *
 * Version 1.1.3
 *
 * Copyright (c) Kieran Greer
 */

package org.jlog2.util;


import java.util.*;


/**
 * This class provides additional String processing functionality over
 * what the CDC Java package 1.0 provides.
 * @author Kieran Greer
 */
public class StringHandler
{
    /**
     * Return true if the String s1 contains the String s2.
     * @param s1 the first String.
     * @param s2 the second String.
     * @return true if s1 contains s2 as part of its sequence of characters.
     */
    public static boolean contains(String s1, String s2)
    {
        if ((s1 != null) && (s2 != null))
        {
            return (s1.indexOf(s2) >= 0);
        }
        
        return false;
    }
    
    /**
     * Return true if the String s1 contains the String s2 ignoring case.
     * @param s1 the first String.
     * @param s2 the second String.
     * @return true if s1 contains s2 as part of its sequence of characters.
     */
    public static boolean containsIgnoreCase(String s1, String s2)
    {
        if ((s1 != null) && (s2 != null))
        {
            s1 = s1.toLowerCase();
            s2 = s2.toLowerCase();
            return (s1.indexOf(s2) >= 0);
        }
        
        return false;
    }

    /**
     * Replace the first occurrence of s2 in s1 with the replacement text.
     * @param s1 the String to change.
     * @param s2 the text to remove.
     * @param replace the text to replace s2 with.
     * @return the s1 String with the first occurrence of s2 replaced with replace.
     */
    public static String replaceFirst(String s1, String s2, String replace)
    {
        int index;                          //index
        String replacedStr;                 //String with replaced text

        if ((s1 != null) && (s2 != null) && (replace != null))
        {
            replacedStr = "";
            
            index = s1.indexOf(s2);
            if (index >= 0)
            {
                replacedStr = s1.substring(0, index);
                replacedStr += replace;
                replacedStr += s1.substring(index + s2.length());
            }
            else
            {
                replacedStr = s1;
            }

            return replacedStr;
        }
        
        return s1;
    }
    
    /**
     * Replace the first occurrence of s2 in s1 with the replacement text. Ignore the case
     * of {@code s2} in {@code s1}, but add {@code replace} as is.
     * @param s1 the String to change.
     * @param s2 the text to remove.
     * @param replace the text to replace s2 with.
     * @return the s1 String with the first occurrence of s2 replaced with replace.
     */
    public static String replaceFirstIgnoreCase(String s1, String s2, String replace)
    {
        int index;                          //index
        String s1lc;                        //String 1 in lower case
        String replacedStr;                 //String with replaced text
        
        if ((s1 != null) && (s2 != null) && (replace != null))
        {
            replacedStr = "";
            s1lc = s1.toLowerCase();
            s2 = s2.toLowerCase();
            
            index = s1lc.indexOf(s2);
            if (index >= 0)
            {
                replacedStr = s1.substring(0, index);
                replacedStr += replace;
                replacedStr += s1.substring(index + s2.length());
            }
            else
            {
                replacedStr = s1;
            }

            return replacedStr;
        }
        
        return s1;
    }

    /**
     * Replace all occurrences of s2 in s1 with the replacement text.
     * @param s1 the String to change.
     * @param s2 the text to remove.
     * @param replace the text to replace s2 with.
     * @return the s1 String with all occurrences of s2 replaced with replace.
     */
    public static String replaceAll(String s1, String s2, String replace)
    {
        int index;                          //index
        String replacedStr;                 //String with replaced text
        String remainingStr;                //String with remaining text

        if ((s1 != null) && (s2 != null) && (replace != null))
        {
            replacedStr = "";
            remainingStr = s1;
            
            index = remainingStr.indexOf(s2);
            if (index >= 0)
            {
                while (index >= 0)
                {
                    replacedStr = remainingStr.substring(0, index);
                    replacedStr += replace;
                    index += s2.length();
                    replacedStr += remainingStr.substring(index);

                    remainingStr = replacedStr;
                    index = (index - s2.length() + replace.length());
                    index = remainingStr.indexOf(s2, index);
                }
            }
            else
            {
                replacedStr = s1;
            }
            
            return replacedStr;
        }

        return s1;
    }
    
    /**
     * Replace all occurrences of s2 in s1 with the replacement text. Ignore the case
     * of {@code s2} in {@code s1}, but add {@code replace} as is.
     * @param s1 the String to change.
     * @param s2 the text to remove.
     * @param replace the text to replace s2 with.
     * @return the s1 String with all occurrences of s2 replaced with replace.
     */
    public static String replaceAllIgnoreCase(String s1, String s2, String replace)
    {
        int index;                          //index
        String s1lc;                        //String 1 in lower case
        String replacedStr;                 //String with replaced text
        String replacedStrlc;               //String with replaced text lower case
        String remainingStr;                //String with remaining text

        if ((s1 != null) && (s2 != null) && (replace != null))
        {
            replacedStr = "";
            remainingStr = s1;
            s1lc = s1.toLowerCase();
            s2 = s2.toLowerCase();
            
            index = s1lc.indexOf(s2);
            if (index >= 0)
            {
                while (index >= 0)
                {
                    replacedStrlc = s1lc.substring(0, index);
                    replacedStrlc += replace;
                    index += s2.length();
                    replacedStrlc += s1lc.substring(index);
                    s1lc = replacedStrlc;
                    
                    index -= s2.length();
                    replacedStr = remainingStr.substring(0, index);
                    replacedStr += replace;
                    index += s2.length();
                    replacedStr += remainingStr.substring(index);
                    remainingStr = replacedStr;
                    
                    index = (index - s2.length() + replace.length());
                    index = s1lc.indexOf(s2, index);
                }
            }
            else
            {
                replacedStr = s1;
            }
            
            return replacedStr;
        }

        return s1;
    }

    /**
     * Tokenize the string based on the separator and return the tokens.
     * @param s1 the string to tokenize.
     * @param separator the character sequence to separate tokens.
     * @param includeSeparator if true include separator string as tokens.
     * @return the list of strings tokenized by the separator.
     */
    public static Vector tokenize(String s1, String separator, boolean includeSeparator)
    {
        int index;                          //index
        String nextToken;                   //next token
        String remainingStr;                //String with remaining text
        Vector tokens;                      //list of tokens

        tokens = new Vector();
        if ((s1 != null) && (separator != null))
        {
            remainingStr = s1;
            index = remainingStr.indexOf(separator);

            if (index >= 0)
            {
                while (index >= 0)
                {
                    nextToken = remainingStr.substring(0, index);
                    tokens.addElement(nextToken);
                    if (includeSeparator) tokens.addElement(separator);

                    index += separator.length();
                    remainingStr = remainingStr.substring(index, remainingStr.length());

                    if (remainingStr.length() > 0)
                        index = remainingStr.indexOf(separator);
                    else
                        index = -1;
                }

                if (remainingStr.length() > 0)
                {
                    if (!remainingStr.equals(separator)) tokens.addElement(remainingStr);
                    else if (includeSeparator) tokens.addElement(remainingStr);
                }
            }
            else
            {
                tokens.addElement(s1);
            }
        }

        return tokens;
    }
}
